# Corporate Calendar Zope product
#
# By Peter Bengtsson, peter@fry-it.com,
# Fry-IT Ltd. London, 2004-2005
# www.fry-it.com
# 
# See: zope.org/Members/malikz/CorpCalendar
#

import re, cgi, os
from pprint import pprint
from string import zfill

from Globals import Persistent, InitializeClass, package_home, DTMLFile
from OFS import SimpleItem, ObjectManager, Folder
from DocumentTemplate import sequence
from AccessControl import ClassSecurityInfo
from zLOG import LOG, ERROR, INFO, PROBLEM, WARNING
from DateTime import DateTime

# Product
from TemplateAdder import addTemplates2Class, CTP
import Utils
from Constants import *


__version__=open(os.path.join(package_home(globals()), 'version.txt')).read().strip()

#----------------------------------------------------------------------------

manage_addCorpCalendarForm = DTMLFile('dtml/addCalendar', globals())

def manage_addCorpCalendar(dispatcher, id, title='', REQUEST=None):
    """ add CorpCalendar instance via the web """
    dest = dispatcher.Destination()
    instance = CorpCalendar(id, title)
    dest._setObject(id, instance)
    self = dest._getOb(id)
    
    if REQUEST is not None:
        # whereto next?
        redirect = REQUEST.RESPONSE.redirect
        redirect(REQUEST.URL1+'/manage_workspace')


#----------------------------------------------------------------------------

class CorpCalendar(Folder.Folder, Persistent):

    meta_type = CORP_CALENDAR_METATYPE

    security=ClassSecurityInfo()
    
    this_package_home = package_home(globals())
    
    
    def __init__(self, id, title=''):
        """ init """
        self.id = id
        self.title = title
        self.ok_formats = OK_FORMATS
        self._user_variables = {}
	

    def getTitle(self):
        """ return title """
        return self.title
    
    def getVersion(self):
	""" return __version__ """
	return __version__

    def getRoot(self):
        """ return root object instance """
        mtype = CORP_CALENDAR_METATYPE
        r = self
        c =0
        while r.meta_type != mtype:
            r = r.aq_parent
            c+=1
            if c>20:
                return None
        return r

    def getRootURL(self):
        """ return absolute_url of getRoot() """
        return self.getRoot().absolute_url()
    
    def getHomeURL(self):
	""" return getRootURL() with extra stuff maybe """
	url = self.getRootURL()
	return url
    

    def showDateNice(self, date, html=0, showyear=1, ofword=0, showdayname=1, fmt=None):
	""" wrap a Utils script """
	if fmt is not None:
	    if Utils.same_type(date, 's'):
		date = DateTime(date)
	    return date.strftime(fmt)
	else:
	    return Utils.showDateNice(date, html, showyear, ofword, showdayname)

    def isListType(self, item):
	""" return if this is a list or tuple """
	return Utils.same_type(item, []) or Utils.same_type(item, ())
    
    def suggestAddTime(self, start_date=None, ampm_format=None, plushours=0):
	""" for the addEvent.dtml form """
	
	if ampm_format is None:
	    ampm_format = int(self.getUserVariable('hour_format')=='ampm')
	
	hr = 10 + plushours
	if start_date is None:
	    if ampm_format:
		return "%sAM"%hr
	    else:
		return "%s:00"%hr
	    
	elif Utils.same_type(start_date, 's'):
	    start_date = DateTime(start_date)
	
	desired = self.ZopeTimeHourless(start_date)
	desired = DateTime("%s %s:00"%(desired.strftime('%Y-%m-%d'), hr))
	today = DateTime()
	while desired < today:
	    desired = desired + (1+plushours)/24.0
	    
	if ampm_format:
	    return "%s%s"%(int(desired.strftime('%I')), desired.strftime('%p'))
	else:
	    return desired.strftime('%H:%M')
	
    def suggestAddTimeEnd(self, start_date=None, ampm_format=None):
	""" almost like suggestAddTime but 1 hour ahead """
	return self.suggestAddTime(start_date, ampm_format=ampm_format, plushours=1)
	
    
    def ZopeTimeHourless(self, datestring=None):
	""" return a DateTime object hourless """
	if datestring is not None:
	    if Utils.same_type(datestring, 's'):
		d = DateTime(datestring)
	    else:
		d = datestring
	else:
	    d = DateTime()
	    
	return DateTime(d.strftime('%Y-%m-%d'))
	
	
	    
    security.declareProtected(ADD_EVENTS_PERMISSION, 'manage_addEvent')
    def manage_addEvent(self, description, submitter,
                        start_date, 
			start_time=None, end_time=None, # 24hr clock
			format=OK_FORMATS[0],
			location='',
			all_day=0,
			same_day=1,
			end_date=None,
			REQUEST=None):
        """ add event object """
        submitter = submitter.strip()
        description = description.strip()
	location = location.strip()
        if not description:
            raise "NoDescription", "Description can't be empty"
        format = format.lower().strip()
        if format not in self.ok_formats:
            format = self.ok_formats[0]
        if Utils.same_type(start_date, 's'):
            start_date = DateTime(start_date)
	    
	# fix booleans
	all_day = int(all_day)
	same_day = int(same_day)
	
	if all_day:
	    start_time = 0
	    end_time = 24
	else:
	    assert start_time, "No start time set"
	    assert end_time, "No end time set"
	    start_time = Utils.parseTimeTo24Hr(start_time)
	    end_time = Utils.parseTimeTo24Hr(end_time)
	
	if same_day:
	    end_date = start_date
	else:
	    if Utils.same_type(end_date, 's'):
		end_date = DateTime(end_date)
	    assert end_date is not None, "No end date set"

	    if end_date < start_date:
		raise "InvalidDates", "End date before start date"

	c=2
        id = origid = start_date.strftime('%a-%d-%b-%y')
        while hasattr(self, id):
            id = origid + "-%s"%c
            c += 1
	    
	today = DateTime()
	
	fmt = '%a %d/%b-%y'
	if submitter:
	    title = "Added by %s on %s"%(submitter, today.strftime(fmt))
	else:
	    title = "Added on %s"%(today.strftime(fmt))
	

        adder = self.manage_addEventObject
        object = adder(id, title, description, format, submitter,
                       start_date, end_date, location,
		       all_day, same_day,
		       start_time, end_time)

        self.rememberUserVariable('format', format)
        self.rememberUserVariable('submitter', submitter)

        if REQUEST is not None:
            url = self.absolute_url()
            params = {}
            if REQUEST.get('date-calendar'):
                params['date-calendar'] = REQUEST.get('date-calendar')

            if REQUEST.get('mode-calendar'):
                params['mode-calendar'] = REQUEST.get('mode-calendar')

            url = Utils.AddParam2URL(url, params)
            REQUEST.RESPONSE.redirect(url)

    security.declareProtected(ADD_EVENTS_PERMISSION, 'manage_addEventObject')
    def manage_addEventObject(self, id, title, 
                        description, format, submitter,
                        start_date, end_date, location, all_day, same_day,
			start_time, end_time):
        """ create object """
	# make sure the start_date and end_date are hourless
	start_date = self.ZopeTimeHourless(start_date)
	end_date = self.ZopeTimeHourless(end_date)
	
        instance = CorpEvent(id, title, description, format, submitter,
                             start_date, end_date, location,
			     all_day, same_day,
			     start_time, end_time)
        self._setObject(id, instance)
        object = self._getOb(id)
        object._preCacheDates()
        return object
    
    security.declareProtected(DELETE_EVENTS_PERMISSION, 'manage_deleteEvent')
    def manage_deleteEvent(self, id, REQUEST=None):
	""" manually delete the object """
	# be sure it exists and all
	root = self.getRoot()
	m = "Not a Corporate event in the calendar root"
	assert id in root.objectIds(CORP_EVENT_METATYPE), m
	self.manage_delObjects([id])
	
        if REQUEST is not None:
            url = root.absolute_url()
            params = {}
            if REQUEST.get('date-calendar'):
                params['date-calendar'] = REQUEST.get('date-calendar')

            if REQUEST.get('mode-calendar'):
                params['mode-calendar'] = REQUEST.get('mode-calendar')

            url = Utils.AddParam2URL(url, params)
            REQUEST.RESPONSE.redirect(url)	
	
    
    
    def changeSetting(self, came_from=None):
	""" search for keys we're ready to work with """
	request = self.REQUEST
	okkeys = ['hour_format','submitter','format']
	if request.QUERY_STRING:
	    all_params = cgi.parse_qs(request.QUERY_STRING)
	else:
	    all_params = {}
	    
	for okkey in okkeys:
	    if request.has_key(okkey):
		self.rememberUserVariable(okkey, request.get(okkey))
		
	if came_from is None:
	    url = request.URL1
	else:
	    url = came_from
	    
	p = '?'
	for k, vlist in all_params.items():
	    if k in ['came_from']+okkeys: # those we can ignore
		continue
	    
	    if '?' in url:
		p = '&amp;'
		
	    for v in vlist:
		url += '%s%s=%s'%(p, k, Utils.url_quote(v))
	    
	request.RESPONSE.redirect(url)
	
	

    def ActionURL(self, url=None):
        """ Returns a url without the ugly 'index_html' bit attached. """
        if url is None:
          url = self.REQUEST.URL
        if url[-11:]=='/index_html':
          return url[:-11]
        else:
          return url
      
    def paramedURL(self, url, params={}):
	""" wrap around Utils.AddParam2URL() but removed
	items from params where values are None. """
	checked_params = {}
	for k, v in params.items():
	    if v is not None:
		checked_params[k] = v
		
	return Utils.AddParam2URL(url, checked_params)
	
      
      
    def changeSettingLink(self, title, **kw):
	""" return a suitable link with <a href """
	cgi_keeps = self.REQUEST.QUERY_STRING
	if cgi_keeps:
	    if cgi_keeps.startswith('&'):
		cgi_keeps = cgi_keeps[1:]
	    cgis = cgi.parse_qs(cgi_keeps)
	else:
	    cgis = {}
	    
	    
	
	uri = 'changeSetting'
	url = self.getRootURL()+'/'+uri
	p = '?'
	for k,vlist in cgis.items():
	    if '?' in url:
		p = '&amp;'
	    for v in vlist:
		url += '%s%s=%s'%(p, k, Utils.url_quote(v))

	for k,v in kw.items():
	    if '?' in url:
		p = '&amp;'
	    try:
		ok = int(v)
		k += ':int'
	    except ValueError:
		pass
	    url += '%s%s=%s'%(p, k, Utils.url_quote(v))
	    
	if '?' in url:
	    p = '&amp;'

	url += '%scame_from=%s'%(p, Utils.url_quote(self.ActionURL()))

	return '<a href="%s">%s</a>'%(url, title)
    
    
    
    def rememberUserVariable(self, key, value):
        """ set in stone """
        cookiekey = '__corpcal_%s'%key.replace(' ','')
        user = self.REQUEST.AUTHENTICATED_USER
        username = user.name
        # cookie
        then = DateTime()+300
        then = then.rfc822()
        response = self.REQUEST.RESPONSE
        response.setCookie(cookiekey, value, expires=then, path='/')
        
        if username.lower().replace(' ','') != 'anonymoususer':
            _user_vs = self._user_variables
            if not self._user_variables.has_key(username):
                _user_vs[username] = {}
                         
            _user_vs[key] = value
            self._user_variables = _user_vs

	    
    def getUserVariable(self, key, default=None):
        """ get from stone """
        cookiekey = '__corpcal_%s'%key.replace(' ','')
        user = self.REQUEST.AUTHENTICATED_USER
        username = user.name
        # cookie
        if self.REQUEST.cookies.has_key(cookiekey):
            return self.REQUEST.cookies.get(cookiekey)
        
        
        if username.lower().replace(' ','') != 'anonymoususer':
            if self._user_variables.has_key(username):
                if self._user_variables[username].has_key(key):
                    return self._user_variables[username][key]
                
        return default


    security.declareProtected('View', 'getAllEvents')
    def getAllEvents(self):
        """ return all event objects """
        return self.objectValues(CORP_EVENT_METATYPE)
    
    def getRecentlyAddedEvents(self, daysback=14, limit=5):
	""" return `limit` event objects no older than
	`daysback`. """
	events = self.getAllEvents()
	checked = []
	today = DateTime()
	for event in events:
	    if today - event.getAddDate() < daysback:
		checked.append(event)
		
	# sort
	checked = sequence.sort(checked, (('add_date',),))
	checked.reverse()
		
	if len(checked) > limit:
	    checked = checked[:limit]
	    
	return checked

    def getRecentlyAddedEventsMarshallable(self, daysback=15, limit=5):
        """ wrap getRecentlyAddedEvents() to return a list of
        dictionaries instead so that it can be marshalled and
        sent over XML-RPC. """
        events = context.getRecentlyAddedEvents(daysback, limit)

        items = []
        iso_fmt = "%Y-%m-%dT%H:%M:%S"
        fixer = Utils.html_entity_fixer
        for event in events:

            d={'url':event.absolute_url(),
               'submitter':event.getSubmitter(),
               'description':fixer(event.getDescription()),
               'title':fixer(event.getTitle()),
               'location':fixer(event.getLocation()),
               'start_time':event.showStartTime(),
               'end_time':event.showEndTime(),
               'start_date':event.getStartDate().strftime(iso_fmt),
               'end_date':event.getEndDate().strftime(iso_fmt),
               'date_posted':event.getAddDate().strftime(iso_fmt),
               }
            items.append(d)

        return items
    

    security.declareProtected('View', 'filterEvents')
    def filterEvents(self, date, events=None):
        """ filter on matching dates """
        if events is None:
            events = self.getAllEvents()
        ok = []
        for event in events:
            if event.onDate(date):
                ok.append(event)

        return ok
    
    def getColorSpectrum(self):
	""" return the colors the calendar view will use """
	colors = ['#CCCCFF','#CCFFCC',
	          '#FFFFCC','#CCFFFF',
	          '#FFCCCC','#FFCCFF',
		  #'#99CC99',
		  '#BDE7BD',
		  '#ABABFF', #'#9999FF',
		  '#AEFDAE','#E9E9C5',
		  '#C8E8E8','#E8C5C5']
	return colors
    
    



    decl = security.declareProtected
    decl('View', 'index_html')
    decl(ADD_EVENTS_PERMISSION, 'addEvent')
    decl('View','standard_html_header')
    decl('View','standard_html_footer')
    decl('View','styles.css')
    
dtmls = ('dtml/index_html',
         'dtml/addEvent',
         'dtml/standard_html_header',
         'dtml/standard_html_footer',
         {'f':'dtml/styles.css','optimize':OPTIMIZE and 'css'},
         )

addTemplates2Class(CorpCalendar, dtmls, extension='dtml')

InitializeClass(CorpCalendar)

#----------------------------------------------------------------------------

manage_addCorpEventForm = DTMLFile('dtml/NotImplemented', globals())

def manage_addCorpEvent(dispatcher, id, *args, **kws):
    """ bad """
    raise "NotImplemented", "Use View part of site instead"


class CorpEvent(SimpleItem.SimpleItem, Persistent):
    """ Event class """

    meta_type = CORP_EVENT_METATYPE

    _properties=({'id':'title',         'type': 'string', 'mode':'w'},
		 {'id':'description',   'type': 'text',   'mode':'w'},
                 {'id':'format',        'type': 'string', 'mode':'w'},
                 {'id':'start_date',    'type': 'date',   'mode':'r'},
                 {'id':'end_date',      'type': 'date',   'mode':'r'},
                 )
	
    # for old objects
    location = ""
    all_day = false
    same_day = false
    title = ""

    security=ClassSecurityInfo()
    def __init__(self, id, title,
                 description, format, submitter,
                 start_date, end_date, location, 
		 all_day, same_day,
		 start_time, end_time):
        """ init """
        self.id = id
	self.title = title
        self.description = description
        self.format = format
        self.submitter = submitter
        self.start_date = start_date
        self.end_date = end_date
	self.location = location.strip()
	self.all_day = int(all_day)
	self.same_day = int(same_day)
	self.start_time = float(start_time)
	self.end_time = float(end_time)
	
        self._datestring_list = []
	self.add_date = DateTime()
	
	
	
    def getId(self):
	""" return id """
	return self.id

    def getTitle(self, maxlength=30):
        """ return parts of the description """
        if len(self.description) > maxlength:
	    return Utils.shorten_carefully(self.description, maxlength)+'...'
        else:
            return self.description

    def getSubmitter(self):
        """ return self.submitter """
        return self.submitter

    def getDescription(self):
        """ return description """
        return self.description

    def showDescription(self):
        """ return description nicely """
        return Utils.ShowDescription(self.getDescription(),
                                     self.getFormat())

    def getFormat(self):
        """ return format """
        return self.format
    
    def getLocation(self):
	""" return self.location """
	return self.location
    
    def showLocation(self):
	""" return location nicely """
	L = self.getLocation()
	return Utils.ShowDescription(L)

    def isSameDay(self):
	""" return same_day """
	return not not self.same_day
    
    def isAllDay(self):
	""" return all_day """
	return not not self.all_day
    
    def getStartDate(self):
        """ return start_date """
	start_date = self.start_date
	if hasattr(self, 'all_day'):
	    if not self.all_day:
		if hasattr(self, 'start_time'):
		    start_time = self.getStartTime()
		else:
		    self._convertMe()
		    start_time = self.getStartTime()
		start_date += start_time/24.0
		    
        return start_date

    def getEndDate(self):
        """ return end_date """
	end_date = self.end_date
	if hasattr(self, 'same_day'):
	    if self.same_day:
		end_date = self.start_date
		
	    if self.all_day:
		end_date += 0.99999
	    else:
		if hasattr(self, 'end_time'):
		    end_time = self.getEndTime()
		else:
		    self._convertMe()
		    end_time = self.getEndTime()
		end_date += end_time/24.0
        return end_date

    def getStartTime(self):
	""" return self.start_time """
	return self.start_time
    
    def getEndTime(self):
	""" return end_time """
	return self.end_time
    
    def _showTime(self, anytime, ampm_format=0):
	""" return nice string of a time """
	hour = int(anytime)
	minute = anytime - hour
	minute = int(round(minute*60))
	if ampm_format:
	    post = 'AM'
	    if hour/12:
		post = 'PM'
	    hour = hour%12
	    if minute:
		return "%s:%s%s"%(hour, zfill(minute,2), post)
	    else:
		return "%s%s"%(hour, post)
	else:
	    return "%s:%s"%(hour, zfill(minute, 2))
	
    def showStartTime(self, ampm_format=None):
	""" return a string of the time (float number) """
	if ampm_format is None:
	    ampm_format = int(self.getUserVariable('hour_format')=='ampm')
	if hasattr(self, 'start_time'):
	    start_time = self.getStartTime()
	else:
	    self._convertMe()
	    start_time = self.getStartTime()
	return self._showTime(start_time, ampm_format)
	
    
    def showEndTime(self, ampm_format=None):
	""" return a string of the time (float number) """
	if ampm_format is None:
	    ampm_format = int(self.getUserVariable('hour_format')=='ampm')
	if hasattr(self, 'end_time'):
	    end_time = self.getEndTime()
	else:
	    self._convertMe()
	    end_time = self.getEndTime()
	
	return self._showTime(end_time, ampm_format)
    
    
    def showDateDifference(self):
	""" return a string of the difference between two dates """
	return Utils.timeSince(self.getStartDate(), self.getEndDate())

    def onDate(self, dateobj):
        """ true if this date is there """
        if Utils.same_type(dateobj, 's'):
            dateobj = DateTime(dateobj)

        return dateobj.strftime('%d%m%Y') in self._datestring_list
    
    def getAddDate(self):
	""" return when the event was added. """
	if hasattr(self, 'add_date'):
	    return self.add_date
	else:
	    self.add_date = self.bobobase_modification_time()
	    return self.bobobase_modification_time()
	
    def _preCacheDates(self):
        """ make a list of strings of the dates for this start_date
        and end_date """
	start_date = self.getStartDate()
	end_date = self.getEndDate()
	
        if int(end_date - start_date) ==0: # same day
            list = [start_date.strftime('%d%m%Y')]
        else:
            list = []
            for i in range(int(end_date - start_date)+1):
                _d = start_date + i
                list.append(_d.strftime('%d%m%Y'))
                
        self._datestring_list = list

    
    security.declareProtected(CHANGE_EVENTS_PERMISSION, 'manage_moveEvent')
    def manage_moveEvent(self, start_date, REQUEST=None):
	""" change the start_date and leave """
	
	# init 
	start_date = start_date.strip()
	
	# first test
	if not start_date:
	    raise "NoStartDate", "No new start date entered"
	
	# try to parse it
	try:
	    start_date = DateTime(start_date)
	except:
	    raise "InvalidDate", "Date input (%s) not recognized"%start_date
	
	# second test
	if start_date > self.end_date:
	    if self.isSameDay():
		self.end_date = start_date
	    else:
		m = "This new start date is greater than the end date (%s)"
		m = m%self.end_date
		raise "CrossingDate", m
	
	# all ok!
	self.start_date = start_date
	
	self._preCacheDates()

        if REQUEST is not None:
            url = self.absolute_url()
            params = {}
            if REQUEST.get('date-calendar'):
                params['date-calendar'] = REQUEST.get('date-calendar')

            if REQUEST.get('mode-calendar'):
                params['mode-calendar'] = REQUEST.get('mode-calendar')

            url = Utils.AddParam2URL(url, params)
            REQUEST.RESPONSE.redirect(url)
	    
	    

    security.declareProtected(CHANGE_EVENTS_PERMISSION, 'manage_editEvent')
    def manage_editEvent(self, description, format, submitter,
                         start_time=None, end_time=None,
			 all_day=None, same_day=None,
			 end_date=None, location=None,
			 REQUEST=None):
        """ edit event object """
        description = description.strip()
        submitter = submitter.strip()
        if not description:
            raise "NoDescription", "Description can't be empty"
        format = format.strip().lower()
        if format not in self.ok_formats:
            format = self.ok_formats[0]

        if Utils.same_type(end_date, 's'):
	    end_date = DateTime(end_date)

	hourlessfunc = self.ZopeTimeHourless
        if hourlessfunc(end_date) < hourlessfunc(self.start_date):
	    m = "End date before start date"
            raise "InvalidDates", m

        # go ahead and change
        self.description = description
        self.submitter = submitter
        self.format = format
	
	if all_day is not None and all_day:
	    self.all_day = true
	    self.start_time = 0
	    self.end_time = 24
	else:
	    assert start_time, "No start time set"
	    assert end_time, "No end time set"
	    start_time = Utils.parseTimeTo24Hr(start_time)
	    end_time = Utils.parseTimeTo24Hr(end_time)
	    assert end_time >= start_time, "End time before start time"
	    self.start_time = start_time
	    self.end_time = end_time
	    

	if same_day is not None:
	    if same_day:
		self.same_day = true
		self.end_date = self.start_date
	    else:
		self.end_date = end_date
		self.same_day = false
	    
	if location is not None:
	    self.location = location.strip()

        self.rememberUserVariable('submitter', submitter)
        self.rememberUserVariable('format', format) 

        self._preCacheDates()

        if REQUEST is not None:
            url = self.getRootURL()
            params = {}
            if REQUEST.get('date-calendar'):
                params['date-calendar'] = REQUEST.get('date-calendar')

            if REQUEST.get('mode-calendar'):
                params['mode-calendar'] = REQUEST.get('mode-calendar')

            url = Utils.AddParam2URL(url, params)
            REQUEST.RESPONSE.redirect(url)

	    
    def getEventURL(self):
	""" return absolute_url() but with some other possible 
	variables. """
	request = self.REQUEST
	url = self.absolute_url()
	params = {}
	if request.get('date-calendar'):
	    params['date-calendar'] = request.get('date-calendar')

	if request.get('mode-calendar'):
	    params['mode-calendar'] = request.get('mode-calendar')

	url = Utils.AddParam2URL(url, params)
	
	return url
	

    def _convertMe(self):
	""" change to the new attributes """
	start_time = self.start_date.strftime('%H:%M')
	start_time = Utils.parseTimeTo24Hr(start_time)
	self.start_time = start_time
	
	end_time = self.end_date.strftime('%H:%M')
	end_time = Utils.parseTimeTo24Hr(end_time)
	self.end_time = end_time
	
	self.start_date = self.ZopeTimeHourless(self.start_date)
	self.end_date = self.ZopeTimeHourless(self.end_date)
	
	if int(start_time)==0 and int(end_time)==0:
	    if self.end_date - self.start_date == 1.0:
		self.all_day = true
		self.same_day = true
		self.end_date = self.start_date
		self.end_time = 24.0

	
	

    decl = security.declareProtected
    decl(VIEW_EVENTS_PERMISSION, 'index_html')
    decl(CHANGE_EVENTS_PERMISSION, 'edit')
    decl(CHANGE_EVENTS_PERMISSION, 'move')
    decl(DELETE_EVENTS_PERMISSION, 'delete')

dtmls = (('dtml/view_event','index_html'),
         ('dtml/editEvent','edit'),
	 ('dtml/moveEvent','move'),
	 ('dtml/deleteEvent','delete'),
         )
addTemplates2Class(CorpEvent, dtmls, extension='dtml')
    
InitializeClass(CorpEvent)    
